import java.io.BufferedReader;
import java.io.FileReader;
import java.io.File;
import java.util.Arrays;
import java.util.Scanner;
import java.io.FileNotFoundException;
import java.time.Duration;
import java.time.Instant;
import java.text.DecimalFormat;

public class Cholesky {
	
 	public static double[][] chol(double[][] a){
		int m = a.length;
		double[][] l = new double[m][m]; //automatically initialzed to 0's
		for(int i = 0; i< m;i++){
			for(int k = 0; k < (i+1); k++){
				double sum = 0;
				for(int j = 0; j < k; j++){
					sum += l[i][j] * l[k][j];
				}
				l[i][k] = (i == k) ? Math.sqrt(a[i][i] - sum) :
					(1.0 / l[k][k] * (a[i][k] - sum));
			}
		}
		return l;
	}
 
	public static void main(String[] args) throws FileNotFoundException{
		
		
		String fileName = "_data/testdata_csv/testdata_" + args[0] + ".csv";
		File f=new File(fileName);
		System.out.println(f.getAbsolutePath());
		try{
		Scanner sc = new Scanner(new BufferedReader(new FileReader(f.getAbsolutePath())));
      		int rows = Integer.parseInt(args[0]);
      		int columns = Integer.parseInt(args[0]);
	      	double [][] matrix = new double[rows][columns];
      		while(sc.hasNextLine()) {
         		for (int i=0; i<matrix.length; i++) {
            			String[] line = sc.nextLine().trim().split(",");
            			for (int j=0; j<line.length; j++) {
               				matrix[i][j] = Float.parseFloat(line[j]);
            			}
         		}
      		}
		
		//System.out.println(Arrays.deepToString(matrix));
		
		//System.out.println(Arrays.deepToString(chol(matrix)));
		
		Instant start_time = Instant.now();
		double[][] chol_out = chol(matrix);
		//System.out.println(Arrays.deepToString(chol(matrix)));
		Instant end_time = Instant.now();
		Duration elapsed_time = Duration.between(start_time, end_time);
		DecimalFormat df = new DecimalFormat();
		df.setMaximumFractionDigits(3);
		double ms = elapsed_time.toNanos();
		ms = ms/1000000;
		System.out.println("Total execution time: "+ df.format(ms) +" ms.");
		
		}
		catch (FileNotFoundException ex)
		{
			System.out.println("data file not found");
		}
					
	}
}
