// CPP program to decompose a matrix using
// Cholesky Decomposition
#include <bits/stdc++.h>
using namespace std;

const int MAX = 64;
void cholesky_geeksforgeeks(double **matrix,int n)
{
	double lower[n][n];
	memset(lower, 0, sizeof(lower));

	// Decomposing a matrix into Lower Triangular
	for (int i = 0; i < n; i++) {
		for (int j = 0; j <= i; j++) {
			double sum = 0;

			if (j == i) // summation for diagonals
			{
				for (int k = 0; k < j; k++)
					sum += pow(lower[j][k], 2);
				lower[j][j] = sqrt(matrix[j][j] - sum);
			} else {
				// Evaluating L(i, j) using L(j, j)
				for (int k = 0; k < j; k++)
					sum += (lower[i][k] * lower[j][k]);
				lower[i][j] = (matrix[i][j] - sum) / lower[j][j];
			}
		}
	}

	// Displaying Lower Triangular and its Transpose
	/*
	for (int i = 0; i < n; i++) {	
		// Lower Triangular
		for (int j = 0; j < n; j++){
			if(i==0 && j==0) cout << "Lower Triangular:" << endl;
			cout << lower[i][j] << "\t";
			if(j==n-1) cout << endl;
		}
		cout << endl;
	}	
*/	
	/*for (int i = 0; i < n; i++) {
		// Transpose of Lower Triangular
		for (int j = 0; j < n; j++){
			if(i==0 && j==0) cout << "Transpose:" << endl;
			cout << lower[j][i] << "\t";
			if(j==n-1) cout << endl;
		}
		cout << endl;
	}*/
}

void read_csv(int row, int col, char *filename, double **data){
	FILE *file;
	file = fopen(filename, "r");

	int i = 0;
	int j = 0;
    char line[64000];
	while (fgets(line, 64000, file) && (i < row))
    {
    	// double row[ssParams->nreal + 1];
        char* tmp = strdup(line);

	    j = 0;
	    const char* tok;
	    for (tok = strtok(line, ","); tok && *tok; j++, tok = strtok(NULL, ",\n"))
	    {
	        data[i][j] = atof(tok);
	        //printf("%f\t", data[i][j]);
	    }
	    //printf("\n");

        free(tmp);
        i++;
    }
	printf("Loaded %dx%d matrix\n", i, j);
}

void main_geeksforgeeks(int n){

	char *filename = (char*)calloc(1024, sizeof(char));
	sprintf(filename, "_data/testdata_csv/testdata_%d.csv", n);
	
	double **matrix = (double**)calloc(n, sizeof(double*));
	for(int i = 0; i < n; i++){
		matrix[i] = (double*)calloc(n, sizeof(double));
	}
	read_csv(n, n, filename, matrix);
	
	auto start_time = chrono::high_resolution_clock::now();	
	cholesky_geeksforgeeks(matrix, n);
	auto end_time = chrono::high_resolution_clock::now();
	chrono::duration<double, milli> elapsed_time = end_time - start_time;
	cout << "Total execution time [Geeksforgeeks]: " << elapsed_time.count() << "ms.\n";

	for(int i = 0; i < n; i++){
		free(matrix[i]);
	}
	free(matrix);
	free(filename);
}


//ROSETTACODE.ORG
#include <math.h>

double *cholesky_rosetta(double *A, int n) {
    double *L = (double*)calloc(n * n, sizeof(double));
    if (L == NULL)
        exit(EXIT_FAILURE);

    for (int i = 0; i < n; i++)
        for (int j = 0; j < (i+1); j++) {
            double s = 0;
            for (int k = 0; k < j; k++)
                s += L[i * n + k] * L[j * n + k];
            L[i * n + j] = (i == j) ?
                           sqrt(A[i * n + i] - s) :
                           (1.0 / L[j * n + j] * (A[i * n + j] - s));
        }

    return L;
}

void show_matrix(double *A, int n) {
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++)
            printf("%2.5f ", A[i * n + j]);
        printf("\n");
    }
    printf("\n");
}

void main_rosetta(int n){
	
	char *filename = (char*)calloc(1024, sizeof(char));
	sprintf(filename, "_data/testdata_csv/testdata_%d.csv", n);
	
	double **matrix_aux = (double**)calloc(n, sizeof(double*));
	for(int i = 0; i < n; i++){
		matrix_aux[i] = (double*)calloc(n, sizeof(double));
	}
	read_csv(n, n, filename, matrix_aux);

	double *matrix = (double*)calloc(n*n, sizeof(double));
	int k = 0;
	for(int i = 0; i < n; i++){
		for(int j = 0; i < n; i++){
			matrix[k] = matrix_aux[i][j];
			k++;
		}
	}

	auto start_time = chrono::high_resolution_clock::now();
	double *c1 = cholesky_rosetta(matrix,n);
	//show_matrix(c1, n);
	auto end_time = chrono::high_resolution_clock::now();
	chrono::duration<double, milli> elapsed_time = end_time - start_time;
	cout << "Total execution time [Rosetta]: " << elapsed_time.count() << "ms.\n";
	free(c1);
}

// Driver Code
int main(int argc, char* argv[])
{
	if(argc != 2){
		printf("Wrong parameters: need 2 ( exe num_diag )\n");
		printf("Introduced parameters: ");
		for(int i = 0; i < argc; i++){
			printf("%s ", argv[i]);
		}
		printf("\n");
		return -1;
	}
	int n = atoi(argv[1]);
	
	main_geeksforgeeks(n);
	
	main_rosetta(n);	
	
	
	return 0;
}

