from __future__ import print_function
 
from pprint import pprint
from math import sqrt

import sys
import time
import csv
import numpy as np

def cholesky_rosetta(A):
    L = [[0.0] * len(A) for _ in range(len(A))]
    for i, (Ai, Li) in enumerate(zip(A, L)):
        for j, Lj in enumerate(L[:i+1]):
            s = sum(Li[k] * Lj[k] for k in range(j))
            Li[j] = sqrt(Ai[i] - s) if (i == j) else \
                      (1.0 / Lj[j] * (Ai[j] - s))
    return L

def cholesky_numpy(A):
    #matrix = np.array(A)
    L = np.linalg.cholesky(A)
    Lprime = L.tolist()
    return Lprime

def main(n):
    filename = "_data/testdata_csv/testdata_%d.csv" % int(n) 
    with open(filename) as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=',')
        x = list(csv_reader)
        matrix = np.array(x).astype("float")

    start_time = time.time()
    Lrosetta = cholesky_rosetta(matrix)
    #pprint(Lrosetta)
    end_time = time.time()
    elapsed_time = end_time - start_time
    print()
    print("Total execution time [Python-Rosetta]: " + str(elapsed_time*1000) + " ms.\n")
    
    start_time = time.time()
    Lnumpy = cholesky_numpy(matrix)
    #pprint(Lnumpy)
    end_time = time.time()
    elapsed_time = end_time - start_time
    print()
    print("Total execution time [Python-Numpy]: " + str(elapsed_time*1000) + " ms.\n")

    Lrosetta = np.round(Lrosetta, 4)
    Lnumpy = np.round(Lnumpy, 4)
    if Lrosetta.all() != Lnumpy.all():
        print("Error: Not machting Cholesky matrixes\n")
    else:
        print("Success: Matching Cholesky matrixes\n")

if __name__ == "__main__":
    main(sys.argv[1]);
    
    
    
